"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var ramda_1 = __importDefault(require("ramda"));
var hasValue = function (n) { return !!n; };
var pointAfter = function (p, a) {
    return p && a &&
        ((p.line === a.line && p.column > a.column) || (p.line > a.line));
};
var pointWithin = function (p, a, b) {
    return pointAfter(p, a) && pointAfter(b, p);
};
exports.byType = function (type) {
    return function (n) {
        return n.type === type;
    };
};
function filterByType(a, astNodeType) { return a.filter(exports.byType(astNodeType)); }
exports.filterByType = filterByType;
exports.byLocation = function (point) {
    return function (n) {
        if (!point || !n || !n.loc) {
            return false;
        }
        var start = n.loc.start;
        var end = n.loc.end;
        return pointWithin(point, start, end);
    };
};
var firstVal = function (a) {
    return a.filter(hasValue).shift();
};
var nodesToDescend = [
    'declaration', 'id', 'init', 'body', 'expression',
    'left', 'right', 'value', 'argument', 'openingElement', 'closingElement',
    'declarations', 'children', 'name',
];
exports.searchBy = function (currentNode, tester, options) {
    if (options === void 0) { options = { lower: false }; }
    if (!currentNode) {
        return;
    }
    var currentResult;
    var lowerResult;
    if (Array.isArray(currentNode)) {
        lowerResult = firstVal(currentNode
            .map(function (n) {
            return exports.searchBy(n, tester, options);
        }));
    }
    else {
        var currentTesterResult = tester(currentNode);
        if (currentTesterResult) {
            // allow testers to return a 'lookahead' node match
            currentResult = (currentTesterResult === true)
                ? currentNode
                : currentTesterResult;
        }
        lowerResult = firstVal(nodesToDescend.map(function (s) {
            var nextDescent = currentNode[s];
            if (nextDescent) {
                if (nextDescent.map) {
                    return firstVal(nextDescent.map(function (d) {
                        return exports.searchBy(d, tester, options);
                    }));
                }
                else {
                    return exports.searchBy(nextDescent, tester, options);
                }
            }
        }));
    }
    var result = options.lower ?
        lowerResult || currentResult :
        currentResult || lowerResult;
    return result;
};
exports.searchByType = function (node, type) {
    return exports.searchBy(node, exports.byType(type));
};
exports.searchByLocation = function (node, point) {
    return exports.searchBy(node, exports.byLocation(point));
};
exports.searchByLocationAndType = function (node, point, type) {
    var bl = exports.byLocation(point);
    var bt = exports.byType(type);
    return exports.searchBy(node, function (n) { return bt(n) && bl(n); }, { lower: true });
};
exports.searchBySuperClass = function (node, superClassName) {
    var tester = function (n) {
        var superClass = n.superClass;
        return exports.byType("ClassDeclaration")(n) &&
            superClass && superClass.name === superClassName;
    };
    return exports.searchBy(node, tester);
};
exports.searchForPropTypes = function (node, componentName) {
    var legacyTester = function (n) {
        var isAssignment = exports.byType('AssignmentExpression')(n);
        var left = n.left;
        if (isAssignment && exports.byType('MemberExpression')(left)) {
            var subjectName = left.object.name;
            var propName = left.property.name;
            if (subjectName === componentName && propName === 'propTypes') {
                var isRightObject = exports.byType('ObjectExpression')(n.right);
                return isRightObject && n.right.properties;
            }
        }
    };
    var staticPropTypesTester = function (n) {
        return exports.byType('ClassProperty')(n) &&
            n.static && ramda_1.default.pathEq(['key', 'name'], 'propTypes', n);
    };
    var legacyMatch = exports.searchBy(node, legacyTester);
    if (legacyMatch) {
        return legacyMatch;
    }
    var classDef = exports.searchBySuperClass(node, 'Component');
    var staticProps = exports.searchBy(classDef, staticPropTypesTester);
    if (staticProps && staticProps.value) {
        return staticProps.value.properties;
    }
    else {
        return [];
    }
};
exports.searchByIdName = function (node, idName) {
    var tester = ramda_1.default.pathEq(['id', 'name'], idName);
    return exports.searchBy(node, tester);
};
exports.printNode = function (node, depth) {
    if (depth === void 0) { depth = 0; }
    var indent = depth || 0;
    if (!node) {
        // console.log('undefined node :(');
        return;
    }
    var pad = ramda_1.default.range(0, indent).map(function () { return '|  '; }).join('');
    var nextDepth = indent + 1;
    var sub = function (s) {
        var next = node[s];
        if (next) {
            if (next.forEach) {
                console.log(pad + s + ':');
                next.forEach(function (n) { return exports.printNode(n, nextDepth); });
            }
            else {
                console.log(pad + s + ':');
                exports.printNode(next, nextDepth);
            }
        }
    };
    var iter = function (s) {
        if (node[s]) {
            console.log(pad + s + ':');
            node[s].forEach(function (n) { return exports.printNode(n, nextDepth); });
        }
    };
    var start = ramda_1.default.path(['loc', 'start'], node);
    var end = ramda_1.default.path(['loc', 'end'], node);
    var fmtLoc = function (loc) {
        return loc && loc.line + "," + loc.column;
    };
    if (node.type) {
        console.log("" + pad + node.type + " " + (node.name || node.operator || '') + " " + fmtLoc(start) + " " + fmtLoc(end));
    }
    else {
        console.log("" + pad + JSON.stringify(node));
    }
    var subs = ['id', 'init', 'body', 'declaration', 'key', 'argument',
        'expression', 'callee', 'object', 'property', 'source', 'local',
        'left', 'right', 'value', 'name',
        'openingElement', 'closingElement'];
    subs.forEach(sub);
    var iters = ['declarations', 'params', 'properties', 'arguments', 'specifiers'];
    iters.forEach(iter);
};
//# sourceMappingURL=node-ops.js.map